--------------------------------------------------
-- Filename: mult.vhd
-- Title: Implementation of a iterative multiplier
-- Author: Hsin-Yun Yao, McGil University
-- Date: August 2004
-------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

Entity mult is
port(
   -- Common ports
   Clk_p : in std_logic;
   Rst_p : in std_logic;
   -- Inputs
   Enable_p : in std_logic;
   Mcand_p : in std_logic_vector(7 downto 0);
   Mlier_p : in std_logic_vector(7 downto 0);
   -- Outputs
   ResValid_p : out std_logic;
   Product_p : out std_logic_vector(15 downto 0)
   );
end mult;

architecture RTL_iter of mult is

signal Cnt8_s : std_logic_vector(3 downto 0);
signal ShiftedMcand_s : std_logic_vector(15 downto 0);
signal ShiftedMlier_s : std_logic_vector(7 downto 0);
signal Product_s : std_logic_vector(15 downto 0);
signal ResValid_s : std_logic;

begin
P1 : process( Clk_p )
begin
   if rising_edge(Clk_p) then
      if Rst_p = '1' then
         -- Reset everything to zero
         Cnt8_s <= (Others => '0');
         ShiftedMcand_s <= (Others => '0');
         ShiftedMlier_s <= (Others => '0');
         Product_s <= (Others => '0');
         ResValid_s <= '0';
         Product_p <= (Others => '0');
      else
         -- Accumulate only if LSB of ShiftedMlier_s is '1'
         if ShiftedMlier_s(0) = '1' then
            Product_s <= ShiftedMcand_s + Product_s;
         end if;
         -- Shifting left, for the next round
         ShiftedMcand_s <= ShiftedMcand_s(14 downto 0) & '0';
         -- Shifting right, removing bits that are done
         ShiftedMlier_s <= '0' & ShiftedMlier_s(7 downto 1);
         if Cnt8_s = "1000" then
            ResValid_s <= '1';
            Product_p <= Product_s;
         else
            Cnt8_s <= Cnt8_s + '1';
         end if;
         -- reads the data in only when Enabled
         if Enable_p = '1' then
            Cnt8_s <= (Others => '0');
            ShiftedMcand_s <= (Others => '0');
            ResValid_s <= '0';
            ShiftedMcand_s(7 downto 0) <= Mcand_p;
            ShiftedMlier_s <= Mlier_p;
            Product_s <= (Others => '0');
         end if;
      end if;
   end if;
end process;

-- Drive outputs
ResValid_p <= ResValid_s;
end RTL_iter;
