--------------------------------------------------
-- Filename: mult_testbench.vhd
-- Title: Automated testbench for the iterative multiplier "mult.vhd"
-- Author: Hsin-Yun Yao, McGil University
-- Date: August 2004
-------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

-- Testbench has not inputs nor outputs
Entity mult_testbench is
end mult_testbench;

architecture behavioural of mult_testbench is
-- This is the DUT (Device Under Test)
component mult
port(
   -- Common ports
   Clk_p : in std_logic;
   Rst_p : in std_logic;
   -- Inputs
   Enable_p : in std_logic;
   Mcand_p : in std_logic_vector(7 downto 0);
   Mlier_p : in std_logic_vector(7 downto 0);
   -- Outputs
   ResValid_p : out std_logic;
   Product_p : out std_logic_vector(15 downto 0)
);
end component;

-- Signals mapping to the DUT in the same order
signal clk_s: std_logic := '0';
signal rst_s: std_logic;
signal enb_s: std_logic;
signal mcd_s: std_logic_vector(7 downto 0);
signal mlr_s: std_logic_vector(7 downto 0);
signal val_s: std_logic;
signal prd_s: std_logic_vector(15 downto 0);
-- 1 -> result is correct, 0 -> incorrect
signal Result_Good : std_logic := '0';

begin
U1: mult port map ( clk_s, rst_s, enb_s, mcd_s, mlr_s, val_s, prd_s);
-- Generate clock and reset signal to facilitate simulation
clk_s <= not clk_s after 20 ns;
rst_s <= '1', '0' after 80 ns;

P1: process
begin
   -- This process runs every time the multiplier finishes its calculation.
   -- make sure everything is synchronous
   wait until rising_edge(clk_s);
   if rst_s = '1' then
      mcd_s <=(Others => '0');
      mlr_s <=(Others => '0');
      enb_s <= '0';
   else
      enb_s <= '1';
      -- increment mlier
      if mlr_s < 255 then
         mlr_s <= mlr_s + 1;
      else
         mlr_s <= (Others=>'0');
      end if;
      -- increment mcand only when the mplier finishes a round
      if mlr_s = "00000000" then
         if mcd_s < 255 then
            mcd_s <= mcd_s + 1;
         else
            mcd_s <= (Others=>'0');
         end if;
      end if;
      -- set enable to zero after one clock cycle
      wait until rising_edge(clk_s);
      enb_s <= '0';
      -- wait for the calculation to be finished
      wait until rising_edge(clk_s) and val_s = '1';
      -- check the results
      if prd_s = mcd_s * mlr_s then
         Result_Good <= '1';
      else
         Result_Good <= '0';
         -- The following line will print error on modelsim command prompt
         assert false report "incorrect" severity Error;
      end if;
    end if;
end process;
end behavioural;
